@@ -22,13 +22,18 @@
 #include "ortp/rtpsession.h"
 #include "rtpsession_priv.h"
 
-
 #if defined(WIN32) || defined(_WIN32_WCE)
 #include "ortp-config-win32.h"
-#else
+#elif HAVE_CONFIG_H
 #include "ortp-config.h" /*needed for HAVE_SYS_UIO_H */
 #endif
 
+#if (_WIN32_WINNT >= 0x0600)
+#include <delayimp.h>
+#undef ExternC
+#include <QOS2.h>
+#endif
+
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #define USE_SENDMSG 1
@@ -282,6 +287,7 @@ rtp_session_set_local_addr (RtpSession * session, const char * addr, int port)
 
 		return 0;
 	}
+	ortp_error("Could not bind RTP socket on port to %s port %i",addr,port);
 	return -1;
 }
 
@@ -318,7 +324,7 @@ int rtp_session_set_multicast_ttl(RtpSession *session, int ttl)
 					 (SOCKET_OPTION_VALUE)	   &session->multicast_ttl, sizeof(session->multicast_ttl));
 
  		} break;
-
+#ifdef ORTP_INET6
         case AF_INET6: {
 
 			retval= setsockopt(session->rtp.socket, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, 
@@ -330,7 +336,7 @@ int rtp_session_set_multicast_ttl(RtpSession *session, int ttl)
 					 (SOCKET_OPTION_VALUE) &session->multicast_ttl, sizeof(session->multicast_ttl));
 
         } break;
-
+#endif
         default:
             retval=-1;
     }
@@ -394,7 +400,7 @@ int rtp_session_set_multicast_loopback(RtpSession *session, int yesno)
 						 (SOCKET_OPTION_VALUE)   &session->multicast_loopback, sizeof(session->multicast_loopback));
 
  		} break;
-
+#ifdef ORTP_INET6
         case AF_INET6: {
 
 			retval= setsockopt(session->rtp.socket, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, 
@@ -406,7 +412,7 @@ int rtp_session_set_multicast_loopback(RtpSession *session, int yesno)
 				 (SOCKET_OPTION_VALUE)	&session->multicast_loopback, sizeof(session->multicast_loopback));
 
         } break;
-
+#endif
         default:
             retval=-1;
     }
@@ -444,6 +450,9 @@ int rtp_session_get_multicast_loopback(RtpSession *session)
 int rtp_session_set_dscp(RtpSession *session, int dscp){
 	int retval=0;
 	int tos;
+#if (_WIN32_WINNT >= 0x0600)
+	OSVERSIONINFOEX ovi;
+#endif
 
 	// Store new DSCP value if one is specified
 	if (dscp>=0) session->dscp = dscp;
@@ -451,27 +460,90 @@ int rtp_session_set_dscp(RtpSession *session, int dscp){
 	// Don't do anything if socket hasn't been created yet
 	if (session->rtp.socket < 0) return 0;
 
-	// DSCP value is in the upper six bits of the TOS field
-	tos = (session->dscp << 2) & 0xFC;
-	switch (session->rtp.sockfamily) {
-		case AF_INET:
-		retval = setsockopt(session->rtp.socket, IPPROTO_IP, IP_TOS, (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
-		break;
+#if (_WIN32_WINNT >= 0x0600)
+	memset(&ovi, 0, sizeof(ovi));
+	ovi.dwOSVersionInfoSize = sizeof(ovi);
+	GetVersionEx((LPOSVERSIONINFO) & ovi);
+
+	ortp_message("check OS support for qwave.lib: %i %i %i\n",
+				ovi.dwMajorVersion, ovi.dwMinorVersion, ovi.dwBuildNumber);
+	if (ovi.dwMajorVersion > 5) {
+
+		if (FAILED(__HrLoadAllImportsForDll("qwave.dll"))) {
+			ortp_warning("Failed to load qwave.dll: no QoS available\n" );
+		}
+		else
+		{
+			if (session->dscp==0)
+				tos=QOSTrafficTypeBestEffort;
+			else if (session->dscp==0x8)
+				tos=QOSTrafficTypeBackground;
+			else if (session->dscp==0x28)
+				tos=QOSTrafficTypeAudioVideo;
+			else if (session->dscp==0x38)
+				tos=QOSTrafficTypeVoice;
+			else
+				tos=QOSTrafficTypeExcellentEffort; /* 0x28 */
+
+			if (session->rtp.QoSHandle==NULL) {
+				QOS_VERSION version;
+				BOOL QoSResult;
+
+				version.MajorVersion = 1;
+				version.MinorVersion = 0;
+
+				QoSResult = QOSCreateHandle(&version, &session->rtp.QoSHandle);
+
+				if (QoSResult != TRUE){
+					ortp_error("QOSCreateHandle failed to create handle with error %d\n",
+						GetLastError());
+					retval=-1;
+				}
+			}
+			if (session->rtp.QoSHandle!=NULL) {
+				BOOL QoSResult;
+				QoSResult = QOSAddSocketToFlow(
+					session->rtp.QoSHandle, 
+					session->rtp.socket,
+					(struct sockaddr*)&session->rtp.rem_addr,
+					tos, 
+					QOS_NON_ADAPTIVE_FLOW, 
+					&session->rtp.QoSFlowID);
+
+				if (QoSResult != TRUE){
+					ortp_error("QOSAddSocketToFlow failed to add a flow with error %d\n", 
+						GetLastError());
+					retval=-1;
+				}
+			}
+		}
+	} else {
+#endif
+		// DSCP value is in the upper six bits of the TOS field
+		tos = (session->dscp << 2) & 0xFC;
+		switch (session->rtp.sockfamily) {
+			case AF_INET:
+			retval = setsockopt(session->rtp.socket, IPPROTO_IP, IP_TOS, (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
+			break;
 #ifdef ORTP_INET6
-	case AF_INET6:
+		case AF_INET6:
 #	ifdef IPV6_TCLASS /*seems not defined by my libc*/
-		retval = setsockopt(session->rtp.socket, IPPROTO_IPV6, IPV6_TCLASS,
-		 (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
+			retval = setsockopt(session->rtp.socket, IPPROTO_IPV6, IPV6_TCLASS,
+			 (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
 #	else
-		/*in case that works:*/
-		retval = setsockopt(session->rtp.socket, IPPROTO_IPV6, IP_TOS,
-		 (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
+			/*in case that works:*/
+			retval = setsockopt(session->rtp.socket, IPPROTO_IPV6, IP_TOS,
+			 (SOCKET_OPTION_VALUE)&tos, sizeof(tos));
+#	endif
+			break;
 #endif
-		break;
-#endif
-	default:
-		retval=-1;
+		default:
+			retval=-1;
+		}
+#if (_WIN32_WINNT >= 0x0600)
 	}
+#endif
+
 	if (retval<0)
 		ortp_warning("Failed to set DSCP value on socket.");
 
@@ -925,17 +997,18 @@ rtp_session_rtp_recv (RtpSession * session, uint32_t user_ts)
 		else
 		{
 		 	int errnum=getSocketErrorCode();
-
-			if (error == 0)
-			{
-				ortp_warning
-					("rtp_recv: strange... recv() returned zero.");
+			if (error == 0 || (error == -1 && errnum==0)){
+				/*0 can be returned by RtpTransport functions in case of EWOULDBLOCK*/
+				/*we ignore it*/
+				/*ortp_warning
+					("rtp_recv: strange... recv() returned zero.");*/
+				/*(error == -1 && errnum==0) for buggy drivers*/
 			}
 			else if (!is_would_block_error(errnum))
 			{
 				if (session->on_network_error.count>0){
 					rtp_signal_table_emit3(&session->on_network_error,(long)"Error receiving RTP packet",INT_TO_POINTER(getSocketErrorCode()));
-				}else ortp_warning("Error receiving RTP packet: %s.",getSocketError());
+				}else ortp_warning("Error receiving RTP packet: %s, err num  [%i],error [%i]",getSocketError(),errnum,error);
 			}
 			/* don't free the cached_mp, it will be reused next time */
 			return -1;	/* avoids an infinite loop ! */
@@ -1013,10 +1086,11 @@ rtp_session_rtcp_recv (RtpSession * session)
 		{
 			int errnum=getSocketErrorCode();
 
-			if (error == 0)
+			if (error == 0 || (error=-1 && errnum==0))
 			{
-				ortp_warning
-					("rtcp_recv: strange... recv() returned zero.");
+				/*ortp_warning
+					("rtcp_recv: strange... recv() returned zero.");*/
+				/*(error == -1 && errnum==0) for buggy drivers*/
 			}
 			else if (!is_would_block_error(errnum))
 			{
