@@ -123,12 +123,6 @@ static char *ipaddr(const StunAddress4 *addr)
    return tmp;
 }
 
-static void
-computeHmac_longterm(char* hmac, const char* input, int length,
-                     const char *username, const char *realm, const char *password);
-static void
-computeHmac_shortterm(char* hmac, const char* input, int length, const char* key);
-
 static bool_t 
 stunParseAtrAddress( char* body, unsigned int hdrLen,  StunAtrAddress4 *result )
 {
@@ -141,8 +135,8 @@ stunParseAtrAddress( char* body, unsigned int hdrLen,  StunAtrAddress4 *result )
    result->family = *body++;
    if (result->family == IPv4Family)
    {
-      UInt16 nport;
-      UInt32 naddr;
+      uint16_t nport;
+      uint32_t naddr;
       memcpy(&nport, body, 2); body+=2;
       result->ipv4.port = ntohs(nport);
 
@@ -183,7 +177,7 @@ stunParseAtrChangeRequest( char* body, unsigned int hdrLen,  StunAtrChangeReques
 static bool_t 
 stunParseAtrError( char* body, unsigned int hdrLen,  StunAtrError *result )
 {
-   if ( hdrLen >= sizeof(StunAtrError) )
+   if ( hdrLen < 4 || hdrLen >= 128+4)
    {
       ortp_error("stun: Incorrect size for SA_ERRORCODE");
       return FALSE;
@@ -326,21 +320,22 @@ turnParseAtrRequestedTransport( char* body, unsigned int hdrLen,  TurnAtrRequest
    return TRUE;
 }
 
-#ifdef ORTP_BIGENDIAN
+#if defined(htonq)
+#elif defined(ORTP_BIGENDIAN)
 #define htonq(n) n
 #define ntohq(n) n
 #else /* little endian */
-static inline UInt64
-htonq (UInt64 v)
+static inline uint64_t
+htonq (uint64_t v)
 {
-  return htonl ((UInt32) (v >> 32))
-    | (UInt64) htonl ((UInt32) v) << 32;
+  return htonl ((uint32_t) (v >> 32))
+    | (uint64_t) htonl ((uint32_t) v) << 32;
 }
-static inline UInt64
-ntohq (UInt64 v)
+static inline uint64_t
+ntohq (uint64_t v)
 {
-  return ntohl ((UInt32) (v >> 32))
-    | (UInt64) ntohl ((UInt32) v) << 32;
+  return ntohl ((uint32_t) (v >> 32))
+    | (uint64_t) ntohl ((uint32_t) v) << 32;
 }
 #endif /* little endian */
 
@@ -358,7 +353,7 @@ turnParseAtrReservationToken( char* body, unsigned int hdrLen,  TurnAtrReservati
 }
 
 static bool_t 
-turnParseAtrFingerprint( char* body, unsigned int hdrLen, TurnAtrFingerprint *result )
+stunParseAtrFingerprint( char* body, unsigned int hdrLen, StunAtrFingerprint *result )
 {
   if ( hdrLen != 4 )
   {
@@ -374,7 +369,7 @@ turnParseAtrFingerprint( char* body, unsigned int hdrLen, TurnAtrFingerprint *re
 static bool_t 
 iceParseAtrPriority( char* body, unsigned int hdrLen, IceAtrPriority *result )
 {
-  if ( hdrLen != sizeof(result) )
+  if ( hdrLen != 4 )
   {
     ortp_error("stun: Incorrect size for ICEA_PRIORITY");
     return FALSE;
@@ -388,7 +383,7 @@ iceParseAtrPriority( char* body, unsigned int hdrLen, IceAtrPriority *result )
 static bool_t 
 iceParseAtrIceControll( char* body, unsigned int hdrLen, IceAtrIceControll *result )
 {
-  if ( hdrLen != sizeof(result) )
+  if ( hdrLen != 8 )
   {
     ortp_error("stun: Incorrect size for ICEA_ICECONTROLLED/ICEA_ICECONTROLLING");
     return FALSE;
@@ -617,7 +612,11 @@ stunParseMessage( char* buf, unsigned int bufLen, StunMessage *msg)
             }
             else
             {
-               ortp_debug("stun: SA_XORMAPPEDADDRESS = %s\n", ipaddr(&msg->xorMappedAddress.ipv4) );
+				uint32_t cookie = 0x2112A442;
+      			uint16_t cookie16 = 0x2112A442 >> 16;
+      			msg->xorMappedAddress.ipv4.port = msg->xorMappedAddress.ipv4.port^cookie16;
+      			msg->xorMappedAddress.ipv4.addr = msg->xorMappedAddress.ipv4.addr^cookie;
+				ortp_debug("stun: SA_XORMAPPEDADDRESS = %s\n", ipaddr(&msg->xorMappedAddress.ipv4) );
             }
       }
       else if (atrType == SA_XORONLY)
@@ -734,7 +733,7 @@ stunParseMessage( char* buf, unsigned int bufLen, StunMessage *msg)
       else if (atrType == SA_FINGERPRINT)
       {
             msg->hasFingerprint = TRUE;
-            if ( turnParseAtrFingerprint(  body,  attrLen,  &msg->fingerprint)== FALSE )
+            if ( stunParseAtrFingerprint(  body,  attrLen,  &msg->fingerprint)== FALSE )
             {
                ortp_error("stun: problem parsing SA_FINGERPRINT\n");
                return FALSE;
@@ -742,7 +741,7 @@ stunParseMessage( char* buf, unsigned int bufLen, StunMessage *msg)
       }
       else if (atrType == ICEA_PRIORITY)
       {
-            msg->hasLifetimeAttributes = TRUE;
+            msg->hasPriority = TRUE;
             if (iceParseAtrPriority(body, attrLen, &msg->priority) == FALSE)
             {
                ortp_error("stun: problem parsing ICEA_PRIORITY");
@@ -794,21 +793,28 @@ stunParseMessage( char* buf, unsigned int bufLen, StunMessage *msg)
 
 
 static char* 
-encode16(char* buf, UInt16 data)
+encode16(char* buf, uint16_t data)
 {
-   UInt16 ndata = htons(data);
-   memcpy(buf, &ndata, sizeof(UInt16));
-   return buf + sizeof(UInt16);
+   uint16_t ndata = htons(data);
+   memcpy(buf, &ndata, sizeof(uint16_t));
+   return buf + sizeof(uint16_t);
 }
 
 static char* 
-encode32(char* buf, UInt32 data)
+encode32(char* buf, uint32_t data)
 {
-   UInt32 ndata = htonl(data);
-   memcpy(buf, &ndata, sizeof(UInt32));
-   return buf + sizeof(UInt32);
+   uint32_t ndata = htonl(data);
+   memcpy(buf, &ndata, sizeof(uint32_t));
+   return buf + sizeof(uint32_t);
 }
 
+static char* 
+encode64(char* buf, uint64_t data)
+{
+   uint64_t ndata = htonq(data);
+   memcpy(buf, &ndata, sizeof(uint64_t));
+   return buf + sizeof(uint64_t);
+}
 
 static char* 
 encode(char* buf, const char* data, unsigned int length)
@@ -819,7 +825,7 @@ encode(char* buf, const char* data, unsigned int length)
 
 
 static char* 
-encodeAtrAddress4(char* ptr, UInt16 type, const StunAtrAddress4 *atr)
+encodeAtrAddress4(char* ptr, uint16_t type, const StunAtrAddress4 *atr)
 {
    ptr = encode16(ptr, type);
    ptr = encode16(ptr, 8);
@@ -843,12 +849,24 @@ encodeAtrChangeRequest(char* ptr, const StunAtrChangeRequest *atr)
 static char* 
 encodeAtrError(char* ptr, const StunAtrError *atr)
 {
+   int padding;
+   int i;
+
    ptr = encode16(ptr, SA_ERRORCODE);
-   ptr = encode16(ptr, 6 + atr->sizeReason);
+   ptr = encode16(ptr, 4 + atr->sizeReason);
    ptr = encode16(ptr, atr->pad);
    *ptr++ = atr->errorClass;
    *ptr++ = atr->number;
    ptr = encode(ptr, atr->reason, atr->sizeReason);
+
+   padding = (atr->sizeReason+4) % 4;
+   if (padding>0)
+   {
+     for (i=0;i<4-padding;i++)
+     {
+       *ptr++ = 0;
+     }
+   }
    return ptr;
 }
 
@@ -867,7 +885,7 @@ encodeAtrUnknown(char* ptr, const StunAtrUnknown *atr)
 }
 
 static char* 
-encodeAtrString(char* ptr, UInt16 type, const StunAtrString *atr)
+encodeAtrString(char* ptr, uint16_t type, const StunAtrString *atr)
 {
    int padding;
    int i;
@@ -897,6 +915,18 @@ encodeAtrIntegrity(char* ptr, const StunAtrIntegrity *atr)
    return ptr;
 }
 
+static char*
+encodeAtrFingerprint(char* ptr, const StunAtrFingerprint *atr)
+{
+	uint32_t val;
+	ptr = encode16(ptr, SA_FINGERPRINT);
+	ptr = encode16(ptr, 4);
+
+	val = atr->fingerprint;
+	val ^= 0x5354554E;
+	ptr = encode32(ptr, val);
+	return ptr;
+}
 
 static char* 
 encodeAtrRequestedTransport(char* ptr, const TurnAtrRequestedTransport *atr)
@@ -927,6 +957,32 @@ encodeAtrDontFragment(char* ptr)
    return ptr;
 }
 
+static char* 
+encodeAtrUseCandidate(char* ptr)
+{
+   ptr = encode16(ptr, ICEA_USECANDIDATE);
+   ptr = encode16(ptr, 0);
+   return ptr;
+}
+
+static char* 
+encodeAtrPriority(char* ptr, const IceAtrPriority *atr)
+{
+   ptr = encode16(ptr, ICEA_PRIORITY);
+   ptr = encode16(ptr, 4);
+   ptr = encode32(ptr, atr->priority);
+   return ptr;
+}
+
+static char* 
+encodeAtrIceControll(char* ptr, uint16_t type, const IceAtrIceControll *atr)
+{
+   ptr = encode16(ptr, type);
+   ptr = encode16(ptr, 8);
+   ptr = encode64(ptr, atr->value);
+   return ptr;
+}
+
 unsigned int
 stunEncodeMessage( const StunMessage *msg, 
                    char* buf, 
@@ -957,8 +1013,7 @@ stunEncodeMessage( const StunMessage *msg,
    {
       ortp_debug("stun: Encoding TA_DONTFRAGMENT: DF\n");
       ptr = encodeAtrDontFragment (ptr);
-   }
-
+   }		  
    if (msg->hasMappedAddress)
    {
       ortp_debug("stun: Encoding SA_MAPPEDADDRESS: %s\n", ipaddr(&msg->mappedAddress.ipv4) );
@@ -1029,6 +1084,28 @@ stunEncodeMessage( const StunMessage *msg,
       ortp_debug("stun: Encoding SA_XORMAPPEDADDRESS: %s\n", ipaddr(&msg->xorMappedAddress.ipv4) );
       ptr = encodeAtrAddress4 (ptr, SA_XORMAPPEDADDRESS, &msg->xorMappedAddress);
    }
+   
+   if (msg->hasPriority)
+   {	   
+      ortp_debug("stun: Encoding ICEA_PRIORITY\n");
+      ptr = encodeAtrPriority (ptr, &msg->priority);
+   }
+   if (msg->hasUseCandidate)
+   {	   
+      ortp_debug("stun: Encoding ICEA_USECANDIDATE\n");
+      ptr = encodeAtrUseCandidate (ptr);
+   }
+   if (msg->hasIceControlled)
+   {	   
+      ortp_debug("stun: Encoding ICEA_ICECONTROLLED\n");
+      ptr = encodeAtrIceControll (ptr, ICEA_ICECONTROLLED, &msg->iceControlled);
+   }
+   if (msg->hasIceControlling)
+   {	   
+      ortp_debug("stun: Encoding ICEA_ICECONTROLLING\n");
+      ptr = encodeAtrIceControll (ptr, ICEA_ICECONTROLLING, &msg->iceControlling);
+   }
+
    if (msg->hasSoftware)
    {
       ortp_debug("stun: Encoding SA_SOFTWARE: %s\n", msg->softwareName.value );
@@ -1043,8 +1120,8 @@ stunEncodeMessage( const StunMessage *msg,
       StunAtrIntegrity integrity;
       //ortp_debug("stun: HMAC with password: %s\n", password->value );
 
-      encode16(lengthp, (UInt16)(ptr - buf - sizeof(StunMsgHdr)+24));
-      computeHmac_longterm(integrity.hash, buf, (int)(ptr-buf) ,
+      encode16(lengthp, (uint16_t)(ptr - buf - sizeof(StunMsgHdr)+24));
+      stunCalculateIntegrity_longterm(integrity.hash, buf, (int)(ptr-buf) ,
         msg->username.value, msg->realmName.value, password->value);
       ptr = encodeAtrIntegrity(ptr, &integrity);
    }
@@ -1055,13 +1132,22 @@ stunEncodeMessage( const StunMessage *msg,
       StunAtrIntegrity integrity;
       //ortp_debug("stun: HMAC with password: %s\n", password->value );
 
-      encode16(lengthp, (UInt16)(ptr - buf - sizeof(StunMsgHdr)+24));
-      computeHmac_shortterm(integrity.hash, buf, (int)(ptr-buf) ,
+      encode16(lengthp, (uint16_t)(ptr - buf - sizeof(StunMsgHdr)+24));
+      stunCalculateIntegrity_shortterm(integrity.hash, buf, (int)(ptr-buf) ,
         password->value);
       ptr = encodeAtrIntegrity(ptr, &integrity);
    }
-	
-   encode16(lengthp, (UInt16)(ptr - buf - sizeof(StunMsgHdr)));
+
+   if (msg->hasFingerprint)
+   {
+      StunAtrFingerprint fingerprint;
+      //ortp_debug("stun: HMAC with password: %s\n", password->value );
+
+      encode16(lengthp, (uint16_t)(ptr - buf - sizeof(StunMsgHdr)+8));
+      fingerprint.fingerprint = stunCalculateFingerprint(buf, (int)(ptr-buf));
+      ptr = encodeAtrFingerprint(ptr, &fingerprint);
+   }
+   encode16(lengthp, (uint16_t)(ptr - buf - sizeof(StunMsgHdr)));
    return (int)(ptr - buf);
 }
 
@@ -1073,7 +1159,7 @@ stunRand(void)
    static bool_t init=FALSE;
    if ( !init )
    { 
-      UInt64 tick;
+      uint64_t tick;
       int seed;
       init = TRUE;
 
@@ -1092,19 +1178,19 @@ stunRand(void)
       tick <<= 32;
       tick |= lowtick;
       }
+#elif defined(__MACH__) 
+	   {
+		   int fd=open("/dev/random",O_RDONLY);
+		   read(fd,&tick,sizeof(tick));
+		   closesocket(fd);
+	   }
 #elif defined(__GNUC__) && ( defined(__i686__) || defined(__i386__) )
       asm("rdtsc" : "=A" (tick));
 #elif defined(__GNUC__) && defined(__amd64__)
       asm("rdtsc" : "=A" (tick));
 #elif defined (__SUNPRO_CC) && defined( __sparc__ )	
       tick = gethrtime();
-#elif defined(__MACH__) 
-      {
-	int fd=open("/dev/random",O_RDONLY);
-	read(fd,&tick,sizeof(tick));
-	closesocket(fd);
-      }
-#elif defined(__linux) || defined(HAVE_DEV_RANDOM) 
+#elif defined(__linux) || defined(__linux__) || defined(HAVE_DEV_RANDOM) 
       {
  	fd_set fdSet;
 	int maxFd=0;
@@ -1177,23 +1263,24 @@ randomPort()
 
 
 #ifdef NOSSL
-static void
-computeHmac_longterm(char* hmac, const char* input, int length,
+void
+stunCalculateIntegrity_longterm(char* hmac, const char* input, int length,
                      const char *username, const char *realm, const char *password)
 {
    strncpy(hmac,"hmac-not-implemented",20);
 }
-static void
-computeHmac_shortterm(char* hmac, const char* input, int length, const char* key)
+void
+stunCalculateIntegrity_shortterm(char* hmac, const char* input, int length, const char* key)
 {
    strncpy(hmac,"hmac-not-implemented",20);
 }
+
 #else
 #include <openssl/hmac.h>
 #include <openssl/md5.h>
 
-static void
-computeHmac_longterm(char* hmac, const char* input, int length,
+void
+stunCalculateIntegrity_longterm(char* hmac, const char* input, int length,
                      const char *username, const char *realm, const char *password)
 {
    unsigned int resultSize=0;
@@ -1209,8 +1296,8 @@ computeHmac_longterm(char* hmac, const char* input, int length,
         (unsigned char*)hmac, &resultSize);
 }
 
-static void
-computeHmac_shortterm(char* hmac, const char* input, int length, const char* key)
+void
+stunCalculateIntegrity_shortterm(char* hmac, const char* input, int length, const char* key)
 {
    unsigned int resultSize=0;
    HMAC(EVP_sha1(), 
@@ -1218,12 +1305,74 @@ computeHmac_shortterm(char* hmac, const char* input, int length, const char* key
         (const unsigned char*) input, length, 
         (unsigned char*)hmac, &resultSize);
 }
+
 #endif
 
-UInt64
+uint32_t
+stunCalculateFingerprint(const char* input, int length)
+{
+	/*-
+	2  *  COPYRIGHT (C) 1986 Gary S. Brown.  You may use this program, or
+	3  *  code or tables extracted from it, as desired without restriction.
+	4  */
+	static uint32_t crc32_tab[] = {
+		0x00000000, 0x77073096, 0xee0e612c, 0x990951ba, 0x076dc419, 0x706af48f,
+		0xe963a535, 0x9e6495a3, 0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988,
+		0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91, 0x1db71064, 0x6ab020f2,
+		0xf3b97148, 0x84be41de, 0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7,
+		0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec, 0x14015c4f, 0x63066cd9,
+		0xfa0f3d63, 0x8d080df5, 0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172,
+		0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b, 0x35b5a8fa, 0x42b2986c,
+		0xdbbbc9d6, 0xacbcf940, 0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59,
+		0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116, 0x21b4f4b5, 0x56b3c423,
+		0xcfba9599, 0xb8bda50f, 0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924,
+		0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d, 0x76dc4190, 0x01db7106,
+		0x98d220bc, 0xefd5102a, 0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433,
+		0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818, 0x7f6a0dbb, 0x086d3d2d,
+		0x91646c97, 0xe6635c01, 0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e,
+		0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457, 0x65b0d9c6, 0x12b7e950,
+		0x8bbeb8ea, 0xfcb9887c, 0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65,
+		0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2, 0x4adfa541, 0x3dd895d7,
+		0xa4d1c46d, 0xd3d6f4fb, 0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0,
+		0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9, 0x5005713c, 0x270241aa,
+		0xbe0b1010, 0xc90c2086, 0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f,
+		0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4, 0x59b33d17, 0x2eb40d81,
+		0xb7bd5c3b, 0xc0ba6cad, 0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a,
+		0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683, 0xe3630b12, 0x94643b84,
+		0x0d6d6a3e, 0x7a6a5aa8, 0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1,
+		0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe, 0xf762575d, 0x806567cb,
+		0x196c3671, 0x6e6b06e7, 0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc,
+		0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5, 0xd6d6a3e8, 0xa1d1937e,
+		0x38d8c2c4, 0x4fdff252, 0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b,
+		0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60, 0xdf60efc3, 0xa867df55,
+		0x316e8eef, 0x4669be79, 0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236,
+		0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f, 0xc5ba3bbe, 0xb2bd0b28,
+		0x2bb45a92, 0x5cb36a04, 0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d,
+		0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a, 0x9c0906a9, 0xeb0e363f,
+		0x72076785, 0x05005713, 0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38,
+		0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21, 0x86d3d2d4, 0xf1d4e242,
+		0x68ddb3f8, 0x1fda836e, 0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777,
+		0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c, 0x8f659eff, 0xf862ae69,
+		0x616bffd3, 0x166ccf45, 0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2,
+		0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db, 0xaed16a4a, 0xd9d65adc,
+		0x40df0b66, 0x37d83bf0, 0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9,
+		0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6, 0xbad03605, 0xcdd70693,
+		0x54de5729, 0x23d967bf, 0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94,
+		0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d
+	};
+	const uint8_t *p = (uint8_t*)input;
+	uint32_t crc;
+
+	crc = ~0U;
+	while (length--)
+		crc = crc32_tab[(crc ^ *p++) & 0xFF] ^ (crc >> 8);
+	return crc ^ ~0U;
+}
+
+uint64_t
 stunGetSystemTimeSecs(void)
 {
-   UInt64 time=0;
+   uint64_t time=0;
 #if	defined(_WIN32) || defined(_WIN32_WCE)
    SYSTEMTIME t;
    /*  CJ TODO - this probably has bug on wrap around every 24 hours */
@@ -1242,9 +1391,9 @@ stunGetSystemTimeSecs(void)
 /* returns TRUE if it scucceeded */
 bool_t 
 stunParseHostName( const char* peerName,
-                   UInt32* ip,
-                   UInt16* portVal,
-                   UInt16 defaultPort )
+                   uint32_t* ip,
+                   uint16_t* portVal,
+                   uint16_t defaultPort )
 {
    struct in_addr sin_addr;
     
@@ -1418,7 +1567,7 @@ stunServerProcessMsg( char* buf,
    StunMessage req;
    StunAddress4 mapped;
    StunAddress4 respondTo;
-   UInt32 flags;
+   uint32_t flags;
    bool_t ok;
    /* set up information for default response */
 	
@@ -1488,12 +1637,6 @@ stunServerProcessMsg( char* buf,
                              "1234", 4, 
                              (const unsigned char*) buf, bufLen-20-4, 
                              hmac, &hmacSize);
-                        /*HMAC(EVP_sha1(), 
-                             "1234", 4, 
-                             reinterpret_cast<const unsigned char*>(buf), bufLen-20-4, 
-                             hmac, &hmacSize);
-                        //assert(hmacSize == 20);
-			*/
                      }
 #endif
 							
@@ -1553,13 +1696,15 @@ stunServerProcessMsg( char* buf,
 		
          if (1) /* do xorMapped address or not */
          {
-            UInt32 cookie = 0x2112A442;
-            UInt16 cookie16;
+            uint32_t cookie = 0x2112A442;
             resp->hasXorMappedAddress = TRUE;
-            cookie16 = ((UInt8*)&cookie)[0]<<8  | ((UInt8*)&cookie)[1];
-            resp->xorMappedAddress.ipv4.port = mapped.port^cookie16;
+            resp->xorMappedAddress.ipv4.port = mapped.port^(cookie>>16);
             resp->xorMappedAddress.ipv4.addr = mapped.addr^cookie;
          }
+
+		resp->hasMappedAddress = TRUE;
+		resp->mappedAddress.ipv4.port=mapped.port;
+		resp->mappedAddress.ipv4.addr=mapped.addr;
          
          resp->hasSourceAddress = TRUE;
          resp->sourceAddress.ipv4.port = (*changePort) ? altAddr->port : myAddr->port;
@@ -1602,8 +1747,8 @@ stunServerProcessMsg( char* buf,
 
          if (req.hasUsername && (req.username.sizeValue > 64 ) )
          {
-            UInt32 source;
-            /* assert( sizeof(int) == sizeof(UInt32) ); */
+            uint32_t source;
+            /* assert( sizeof(int) == sizeof(uint32_t) ); */
 					
             sscanf(req.username.value, "%x", &source);
             resp->hasReflectedFrom = TRUE;
@@ -1730,7 +1875,7 @@ stunStopServer(StunServerInfo *info)
 }
 
 int 
-stunFindLocalInterfaces(UInt32* addresses,int maxRet)
+stunFindLocalInterfaces(uint32_t* addresses,int maxRet)
 {
 #if defined(WIN32) || defined(_WIN32_WCE) || defined(__sparc__)
    return 0;
@@ -1760,7 +1905,7 @@ stunFindLocalInterfaces(UInt32* addresses,int maxRet)
       struct sockaddr a;
       struct sockaddr_in* addr;
    
-      UInt32 ai;
+      uint32_t ai;
       int si = sizeof(ifr->ifr_name) + sizeof(struct sockaddr);
       tl -= si;
       ptr += si;
@@ -1926,7 +2071,7 @@ stunTest( StunAddress4 *dest, int testNum, StunAddress4* sAddr , StunAddress4 *s
    /* assert( dest.port != 0 ); */
 	
    int port = randomPort();
-   UInt32 interfaceIp=0;
+   uint32_t interfaceIp=0;
    Socket myFd;
    StunAtrString username;
    StunAtrString password;
@@ -2011,7 +2156,7 @@ stunNatType( StunAddress4 *dest,
 { 
    /* assert( dest.addr != 0 ); */
    /* assert( dest.port != 0 ); */
-   UInt32 interfaceIp=0;
+   uint32_t interfaceIp=0;
    Socket myFd1;
    Socket myFd2;
 
@@ -2030,8 +2175,8 @@ stunNatType( StunAddress4 *dest,
    StunAtrString username;
    StunAtrString password;
    int count=0;
-   UInt64 second_started;
-   UInt64 second_elapsed;
+   uint64_t second_started;
+   uint64_t second_elapsed;
    Socket s;
 
    if ( hairpin ) 
@@ -2401,10 +2546,8 @@ stunOpenSocket( StunAddress4 *dest, StunAddress4* mapAddr,
 
    if (resp.hasXorMappedAddress==TRUE)
    {
-      UInt32 cookie = 0x2112A442;
-      UInt16 cookie16 = 0x2112A442 >> 16;
-      mappedAddr.port = resp.xorMappedAddress.ipv4.port^cookie16;
-      mappedAddr.addr = resp.xorMappedAddress.ipv4.addr^cookie;
+      mappedAddr.port = resp.xorMappedAddress.ipv4.port;
+      mappedAddr.addr = resp.xorMappedAddress.ipv4.addr;
    }
    else
      mappedAddr = resp.mappedAddress.ipv4;
@@ -2511,10 +2654,8 @@ stunOpenSocketPair(StunAddress4 *dest,
 	  
       if (resp.hasXorMappedAddress==TRUE)
       {
-        UInt32 cookie = 0x2112A442;
-        UInt16 cookie16 = 0x2112A442 >> 16;
-        mappedAddr[i].port = resp.xorMappedAddress.ipv4.port^cookie16;
-        mappedAddr[i].addr = resp.xorMappedAddress.ipv4.addr^cookie;
+        mappedAddr[i].port = resp.xorMappedAddress.ipv4.port;
+        mappedAddr[i].addr = resp.xorMappedAddress.ipv4.addr;
       }
       else
         mappedAddr[i] = resp.mappedAddress.ipv4;
@@ -2703,10 +2844,8 @@ turnAllocateSocketPair(StunAddress4 *dest,
       {
         if (resp.hasXorRelayedAddress==TRUE)
         {
-          UInt32 cookie = 0x2112A442;
-          UInt16 cookie16 = 0x2112A442 >> 16;
-          mappedAddr[i].port = resp.xorRelayedAddress.ipv4.port^cookie16;
-          mappedAddr[i].addr = resp.xorRelayedAddress.ipv4.addr^cookie;
+          mappedAddr[i].port = resp.xorRelayedAddress.ipv4.port;
+          mappedAddr[i].addr = resp.xorRelayedAddress.ipv4.addr;
         }
         else
         {
